"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
module.exports = void 0;

var _pluginutils = require("@rollup/pluginutils");

var _magicString = _interopRequireDefault(require("magic-string"));

var _nodePath = _interopRequireDefault(require("node:path"));

var rdt = _interopRequireWildcard(require("react-docgen-typescript"));

var telejson = _interopRequireWildcard(require("telejson"));

var _constants = require("./constants.cjs");

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const plugin = ({
  apply,
  componentNameResolver,
  customComponentTypes = [],
  enforce = "pre",
  exclude = ["**/**.stories.tsx"],
  handler,
  include = ["**/**.tsx"],
  propFilter = prop => !prop.parent?.fileName.includes("node_modules"),
  savePropValueAsString = false,
  shouldExtractLiteralValuesFromEnum = false,
  shouldExtractValuesFromUnion = false,
  shouldIncludeExpression = false,
  shouldIncludePropTagMap = true,
  shouldRemoveUndefinedFromOptional = true,
  skipChildrenPropWithoutDoc = true,
  sourcemap = true,
  tsconfig = _nodePath.default.resolve("tsconfig.json")
} = {}) => {
  const filter = (0, _pluginutils.createFilter)(include, exclude);
  const parser = rdt.withCustomConfig(tsconfig, {
    componentNameResolver,
    customComponentTypes,
    propFilter,
    savePropValueAsString,
    shouldExtractLiteralValuesFromEnum,
    shouldExtractValuesFromUnion,
    shouldIncludeExpression,
    shouldIncludePropTagMap,
    shouldRemoveUndefinedFromOptional,
    skipChildrenPropWithoutDoc
  });
  return {
    apply,
    enforce,
    name: _constants.PLUGIN_NAME,
    transform: async (code, id) => {
      if (!filter(id)) return;
      const docs = parser.parse(id);
      if (docs.length === 0) return null;
      const src = new _magicString.default(code);

      for (let doc of docs) {
        if (handler) doc = await handler(doc, code, id);

        const __docgenInfo = telejson.stringify(doc);

        src.append(`
;${doc.displayName}.__docgenInfo=${__docgenInfo}`);
      }

      if (sourcemap === false) return src.toString();
      return {
        code: src.toString(),
        map: src.generateMap({ ...(sourcemap === true ? {
            hires: true
          } : sourcemap),
          source: id
        })
      };
    }
  };
};

var _default = plugin;
module.exports = _default;