import { createFilter } from "@rollup/pluginutils";
import MagicString from "magic-string";
import path from "node:path";
import * as rdt from "react-docgen-typescript";
import * as telejson from "telejson";
import { PLUGIN_NAME } from "./constants.mjs";
const plugin = ({
  apply,
  componentNameResolver,
  customComponentTypes = [],
  enforce = "pre",
  exclude = ["**/**.stories.tsx"],
  handler,
  include = ["**/**.tsx"],
  propFilter = (prop) => !prop.parent?.fileName.includes("node_modules"),
  savePropValueAsString = false,
  shouldExtractLiteralValuesFromEnum = false,
  shouldExtractValuesFromUnion = false,
  shouldIncludeExpression = false,
  shouldIncludePropTagMap = true,
  shouldRemoveUndefinedFromOptional = true,
  skipChildrenPropWithoutDoc = true,
  sourcemap = true,
  tsconfig = path.resolve("tsconfig.json")
} = {}) => {
  const filter = createFilter(include, exclude);
  const parser = rdt.withCustomConfig(tsconfig, {
    componentNameResolver,
    customComponentTypes,
    propFilter,
    savePropValueAsString,
    shouldExtractLiteralValuesFromEnum,
    shouldExtractValuesFromUnion,
    shouldIncludeExpression,
    shouldIncludePropTagMap,
    shouldRemoveUndefinedFromOptional,
    skipChildrenPropWithoutDoc
  });
  return {
    apply,
    enforce,
    name: PLUGIN_NAME,
    transform: async (code, id) => {
      if (!filter(id))
        return;
      const docs = parser.parse(id);
      if (docs.length === 0)
        return null;
      const src = new MagicString(code);
      for (let doc of docs) {
        if (handler)
          doc = await handler(doc, code, id);
        const __docgenInfo = telejson.stringify(doc);
        src.append(`
;${doc.displayName}.__docgenInfo=${__docgenInfo}`);
      }
      if (sourcemap === false)
        return src.toString();
      return {
        code: src.toString(),
        map: src.generateMap({
          ...sourcemap === true ? { hires: true } : sourcemap,
          source: id
        })
      };
    }
  };
};
export default plugin;
